// ***********************************************************
// Project: MegaDrum
// Author: Dmitri Skachkov
// Module description:
// ***********************************************************
#include <p18cxxx.h>
#include <usart.h>
#include "usb_config.h"
#include "USB\usb.h"

#define FCPU			48000000
#define BAUDRATE31K		31250
#define BAUDRATE125K	125000
#define BAUDRATE187K	187500
#define BAUDRATE250K	250000

#if defined(__18F2550) || defined(__18F2450)
//		#pragma config OSC = HS
		//#pragma config PLLDIV   = 1         // (4 MHz crystal for 4MHz devider)
		//#pragma config PLLDIV   = 2         // (8 MHz crystal for 4MHz devider)
		#pragma config PLLDIV   = 3         // (12 MHz crystal for 4MHz devider)
		//#pragma config PLLDIV   = 4         // (16 MHz crystal for 4MHz devider)
		//#pragma config PLLDIV   = 5         // (20 MHz crystal for 4MHz devider)
        #pragma config CPUDIV   = OSC1_PLL2   
        #pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
        #pragma config FOSC     = HSPLL_HS
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = OFF
        #pragma config BOR      = ON
#if defined(__18F2550)
        #pragma config BORV     = 3
#else
        #pragma config BORV     = 43
#endif
        #pragma config VREGEN   = ON      //USB Voltage Regulator
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = ON
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
//      #pragma config CCP2MX   = ON
        #pragma config STVREN   = ON
        #pragma config LVP      = ON
//      #pragma config ICPRT    = OFF       // Dedicated In-Circuit Debug/Programming
        #pragma config XINST    = OFF       // Extended Instruction Set
        #pragma config CP0      = OFF
        #pragma config CP1      = OFF
//      #pragma config CP2      = OFF
//      #pragma config CP3      = OFF
        #pragma config CPB      = OFF
//      #pragma config CPD      = OFF
        #pragma config WRT0     = OFF
        #pragma config WRT1     = OFF
//      #pragma config WRT2     = OFF
//      #pragma config WRT3     = OFF
        #pragma config WRTB     = OFF       // Boot Block Write Protection
        #pragma config WRTC     = OFF
//      #pragma config WRTD     = OFF
        #pragma config EBTR0    = OFF
        #pragma config EBTR1    = OFF
//      #pragma config EBTR2    = OFF
//      #pragma config EBTR3    = OFF
        #pragma config EBTRB    = OFF
#endif

#if defined(__18F14K50) || defined(__18F13K50)

        #pragma config CPUDIV = NOCLKDIV, USBDIV = OFF                   // CONFIG1L
        #pragma config FOSC = HS, PLLEN=ON, FCMEN = OFF, IESO = OFF                     // CONFIG1H
        #pragma config PWRTEN = OFF, BOREN = OFF, BORV = 30					// CONFIG2L
        #pragma config WDTEN = OFF, WDTPS = 32768                                     // CONFIG2H
        #pragma config MCLRE = ON, HFOFST = OFF				        // CONFIG3H
        #pragma config STVREN = ON, LVP = ON, XINST = OFF, BBSIZ=OFF	// CONFIG4L
        #pragma config CP0 = OFF, CP1 = OFF						                    // CONFIG5L
        #pragma config CPB = OFF                                    			    // CONFIG5H
        #pragma config WRT0 = OFF, WRT1 = OFF						                // CONFIG6L
        //#pragma config WRTB = ON, WRTC = OFF                            			// CONFIG6H
        // Disabled WRTB for debugging.  Reenable for real.
        #pragma config WRTB = OFF, WRTC = OFF                            			// CONFIG6H
        #pragma config EBTR0 = OFF, EBTR1 = OFF							           	// CONFIG7L
        #pragma config EBTRB = OFF                                                  // CONFIG7H
        
#endif

/*
typedef struct {
	unsigned char	buffer[4];
	unsigned char	pointer;
	unsigned char	max_data;
	unsigned char	state;
	unsigned char	full;
} MidiOutData;
*/

//extern MidiOutData midi_out_data[NUMBER_MIDI_INTERFACES];
extern void USBTxMIDI(unsigned char c);
extern void USBTxClearRingBuffer(void);
extern void clear_usb_hack(void);
extern unsigned char USBTxBusy(void);
extern unsigned char USBRxMIDI(void);
extern void USBDeviceInit(void);
extern void USBUserInit(void);
extern void USBDeviceTasks(void);
extern unsigned char USBRxReady(void);

extern unsigned char USBUSERConfigured;

/*
unsigned char rx_out_pointer[NUMBER_MIDI_INTERFACES] = {
	0,
#if NUMBER_MIDI_INTERFACES > 1
	0,
#endif
#if NUMBER_MIDI_INTERFACES > 2
	0,
#endif
#if NUMBER_MIDI_INTERFACES > 3
	0,
#endif
};
*/
void HighPriorityISRCode (void);

#pragma code HIGH_INTERRUPT_VECTOR = 0x8
void High_ISR (void)
{
	_asm goto HighPriorityISRCode _endasm
}

#pragma code

#pragma interrupt HighPriorityISRCode
void HighPriorityISRCode (void) {
	unsigned char c;
	unsigned char status;

	if (PIR1bits.RCIF) {

		status = RCSTA;
		c = RCREG;
		USBTxMIDI(c);
	}
	#if defined(USB_INTERRUPT)
	//if (PIR2bits.USBIF) {
		USBDeviceTasks();
	//	PIR2bits.USBIF = 0;
	//}
	#endif
}

void getUSBMIDI(void) {
	unsigned char c;
	c = USBRxMIDI();
	while (!PIR1bits.TXIF);
	WriteUSART(c);
}

void usart_init(unsigned char high_speed){
	unsigned short baud;
	
	switch(high_speed) {
		case 1:
			baud = (FCPU/16/BAUDRATE125K)-1;
			break;
		case 2:
			baud = (FCPU/16/BAUDRATE187K)-1;
			break;
		case 3:
			baud = (FCPU/16/BAUDRATE250K)-1;
			break;
		default:
			baud = (FCPU/16/BAUDRATE31K)-1;
			break;

	}
   	OpenUSART(
		USART_TX_INT_OFF  &
           USART_RX_INT_ON  &
           USART_ASYNCH_MODE &
           USART_EIGHT_BIT   &
           USART_CONT_RX     &
           USART_BRGH_HIGH,
           baud
		);

	PIE1bits.RCIE = 1; 
}

/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *
 * Note:            None
 *******************************************************************/
static void USBInitializeSystem(void)
{
        
    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
#if defined(__18F14K50) || defined(__18F13K50)
	ANSELH	= 0;
	ANSEL	= 0;
#endif

	usart_init(0);	// init it originally for low speed = 31250

	/* Enable interrupt priority */
	RCONbits.IPEN = 1;
	/* Make receive interrupt high priority */
	IPR1bits.RCIP = 1;

	INTCONbits.PEIE	= 1;
	INTCONbits.GIE	= 1;	

}//end InitializeSystem

// ***********************************************************
// Main program
//
void main(void) {

	USBInitializeSystem();	// Initialize USB
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

	clear_usb_hack();
	while (1){
        	#if defined(USB_POLLING)
				//USBDeviceTasks();
			#endif
			if (USBRxReady()) {
				getUSBMIDI();
			}
			//PIE1bits.RCIE = 0;		//Disable USART RX interrupt to block re-entry into USBTxClearRingBuffer()
			USBTxClearRingBuffer();
			//USBTxPulse();
			//PIE1bits.RCIE = 1; 
	}
}
