/********************************************************************************
  File Information:
    FileName:       usb_function_cdc.c
    Dependencies:   See INCLUDES section
    Processor:      PIC18 or PIC24 USB Microcontrollers
    Hardware:       The code is natively intended to be used on the following
                    hardware platforms: PICDEM FS USB Demo Board,
                    PIC18F87J50 FS USB Plug-In Module, or
                    Explorer 16 + PIC24 USB PIM.  The firmware may be
                    modified for use on other USB platforms by editing the
                    HardwareProfile.h file.
    Complier:   Microchip C18 (for PIC18) or C30 (for PIC24)
    Company:        Microchip Technology, Inc.

    Software License Agreement:

    The software supplied herewith by Microchip Technology Incorporated
    (the Company) for its PIC Microcontroller is intended and
    supplied to you, the Companys customer, for use solely and
    exclusively on Microchip PIC Microcontroller products. The
    software is owned by the Company and/or its supplier, and is
    protected under applicable copyright laws. All rights are reserved.
    Any use in violation of the foregoing restrictions may subject the
    user to criminal sanctions under applicable laws, as well as to
    civil liability for the breach of the terms and conditions of this
    license.

    THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
    WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
    TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
    PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
    IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
    CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
    
  Summary:
    This file contains all of functions, macros, definitions, variables,
    datatypes, etc. that are required for usage with the CDC function
    driver. This file should be included in projects that use the CDC
    \function driver.
    
    
    
    This file is located in the "\<Install Directory\>\\Microchip\\USB\\CDC
    Device Driver" directory.
  Description:
    USB CDC Function Driver File
    
    This file contains all of functions, macros, definitions, variables,
    datatypes, etc. that are required for usage with the CDC function
    driver. This file should be included in projects that use the CDC
    \function driver.
    
    This file is located in the "\<Install Directory\>\\Microchip\\USB\\CDC
    Device Driver" directory.
    
    When including this file in a new project, this file can either be
    referenced from the directory in which it was installed or copied
    directly into the user application folder. If the first method is
    chosen to keep the file located in the folder in which it is installed
    then include paths need to be added so that the library and the
    application both know where to reference each others files. If the
    application folder is located in the same folder as the Microchip
    folder (like the current demo folders), then the following include
    paths need to be added to the application's project:
    
    ..\\Include
    
    ..\\..\\Include
    
    ..\\..\\Microchip\\Include
    
    ..\\..\\\<Application Folder\>
    
    ..\\..\\..\\\<Application Folder\>
    
    If a different directory structure is used, modify the paths as
    required. An example using absolute paths instead of relative paths
    would be the following:
    
    C:\\Microchip Solutions\\Microchip\\Include
    
    C:\\Microchip Solutions\\My Demo Application                                 
  ********************************************************************************/

/** I N C L U D E S **********************************************************/
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "usb_config.h"
#include "USB\usb.h"
#include "usb_function_audio.h"
#include "HardwareProfile.h"

/** V A R I A B L E S ********************************************************/
/** V A R I A B L E S ********************************************************/
#if defined(__18F14K50) || defined(__18F13K50) || defined(__18LF14K50) || defined(__18LF13K50) 
    #pragma udata usbram2
#elif defined(__18F2455) || defined(__18F2550) || defined(__18F4455) || defined(__18F4550)\
    || defined(__18F2458) || defined(__18F2453) || defined(__18F4558) || defined(__18F4553)
    #pragma udata USB_VARIABLES=0x500
#elif defined(__18F2450)|| defined(__18F4450) // Added 2/13/09 - PES
	#pragma udata USB_VARIABLES=0x4A0
#else
    #pragma udata
#endif

volatile FAR unsigned char usb_data_rx[MIDI_DATA_OUT_EP_SIZE];
volatile FAR unsigned char midi_out_buffer[4*NUMBER_MIDI_INTERFACES];
volatile FAR unsigned char usb_out_buffer[4*NUMBER_MIDI_INTERFACES];

#pragma udata

volatile FAR unsigned char midi_data_rx[MIDI_DATA_OUT_EP_SIZE];

unsigned char midi_data_rx_pointer = 1;

BYTE midi_data_rx_len;            // total rx length
BYTE midi_rx_len;            // total rx length

USB_HANDLE MIDIDataOutHandle;
USB_HANDLE MIDIDataInHandle;

#define MIDI_OUT_BUFFER_STATE_EMPTY		0
#define MIDI_OUT_BUFFER_STATE_FULL		1
#define MIDI_OUT_BUFFER_STATE_SENDING	2
unsigned char midi_out_state = 0;
unsigned char midi_out_max_data = 0;
unsigned char midi_out_pointer = 0;

unsigned char midi_in_data_max = 0;
unsigned char midi_in_received = 0;

unsigned char ring_buffer[20];
unsigned char ring_buffer_in = 0;
unsigned char ring_buffer_out = 0;
unsigned char ring_buffer_outstanding = 0;
unsigned char midi_out_buffer_full = 0;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void USBTxMIDI(unsigned char c);

/** D E C L A R A T I O N S **************************************************/
//#pragma code

/** C L A S S  S P E C I F I C  R E Q ****************************************/



/** U S E R  A P I ***********************************************************/


/**************************************************************************
  Function:
        void CDCInitEP(void)
    
  Summary:
    This function initializes the CDC function driver. This function should
    be called after the SET_CONFIGURATION command.
  Description:
    This function initializes the CDC function driver. This function sets
    the default line coding (baud rate, bit parity, number of data bits,
    and format). This function also enables the endpoints and prepares for
    the first transfer from the host.
    
    This function should be called after the SET_CONFIGURATION command.
    This is most simply done by calling this function from the
    USBCBInitEP() function.
    
    Typical Usage:
    <code>
        void USBCBInitEP(void)
        {
            CDCInitEP();
        }
    </code>
  Conditions:
    None
  Remarks:
    None                                                                   
  **************************************************************************/
void MIDIInitEP(void)
{ 
	unsigned char i;
    midi_rx_len = 0;

    USBEnableEndpoint(MIDI_DATA_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);

	MIDIDataOutHandle = USBRxOnePacket(MIDI_DATA_EP,(BYTE*)&usb_data_rx,sizeof(usb_data_rx));
	MIDIDataInHandle = NULL;

}//end MIDIInitEP

/**********************************************************************************
* Decode incoming MIDI stream of date and pack it into buffer,
* c - incoming data
* returns > 0 if buffer is filled and ready
**********************************************************************************/
unsigned char MidiStreamToUSB(unsigned char c) {
	unsigned char result = MIDI_OUT_BUFFER_STATE_EMPTY;

	if (c&0x80) {
		midi_out_state |= MIDI_STARTED;
		switch(c&0xf0) {
			case 0x80:	// Note Off
			case 0x90:	// Note On
			case 0xa0:	// Polyphonic Key Pressure
			case 0xb0:	// Control Change
			case 0xe0:	// Pitch Bend
				midi_out_buffer[0] = (c>>4);
				midi_out_buffer[1] = c;
				midi_out_pointer = 2;
				midi_out_max_data = 4;
				break;
			case 0xc0:	// Program Change
			case 0xd0:	// Channel Pressure
				midi_out_buffer[0] = (c>>4);
				midi_out_buffer[1] = c;
				midi_out_buffer[3] = 0;
				midi_out_pointer = 2;
				midi_out_max_data = 3;
				break;
			default:	// only 0xf0 left
				switch(c) {
					case 0xf0:
						midi_out_state |= MIDI_SYSEX;
						midi_out_buffer[0] = 0x04;
						midi_out_buffer[1] = c;
						midi_out_buffer[3] = 0;
						midi_out_pointer = 2;
						midi_out_max_data = 4;
						break;
					case 0xf7:
						if(midi_out_state&MIDI_SYSEX) {
							//Complete Sysex only if it was started
							midi_out_buffer[0] = (midi_out_pointer)+4;
							midi_out_buffer[midi_out_pointer] = c;
							while (midi_out_pointer<3) {
								midi_out_pointer++;
								midi_out_buffer[midi_out_pointer] = 0;
							}
							result = MIDI_OUT_BUFFER_STATE_FULL;
						}
						midi_out_state = 0;
						break;
					case 0xf1:
						// MegaDrum specific command
						midi_out_state |= MIDI_MEGADRUM;
						midi_out_buffer[0] = 0xf1;
						midi_out_pointer = 1;
						midi_out_max_data = 4;
						break;
					case 0xf2:
						if(midi_out_state&MIDI_MEGADRUM) {
							// Complete MegaDrum command only if it was started
							if (midi_out_pointer == 3) {
								if (midi_out_buffer[1] == MEGADRUM_SPEED) {
									//MegaDrum command to change MIDI (USART) speed
									if (midi_out_buffer[2] < 4) {
										// 0 - 31250
										// 1 - 125000
										// 2 - 250000
										usart_init(midi_out_buffer[2]);
									}
								}
								if (midi_out_buffer[1] == MEGADRUM_ECHO) {
									//MegaDrum echo request
									WriteUSART(0xf1);
									WriteUSART(MEGADRUM_ECHO);
									WriteUSART(midi_out_buffer[2]);
									WriteUSART(0xf2);
								}
							}
						}
						midi_out_state = 0;
						break;
/*
					case 0xf3:
						usart_init(0);
						midi_out_state = 0;
						break;
					case 0xf4:
						usart_init(1);
						midi_out_state = 0;
						break;
					case 0xf5:
						usart_init(2);
						midi_out_state = 0;
						break;
					case 0xf6:
						usart_init(3);
						midi_out_state = 0;
*/
						break;
					default:	// Other 0xfx messages are not implemented and not needed for MegaDrum
						midi_out_state = 0;
						break;
				}
				break;
		}
	} else {
		// Continue Sysex or MIDI
		if(midi_out_state) {
			midi_out_buffer[midi_out_pointer] = c;
			midi_out_pointer++;
			if (midi_out_pointer >= midi_out_max_data) {
				if (midi_out_state&MIDI_SYSEX) {
					//midi_out_buffer[0] = 0x04;
					midi_out_pointer = 1;
					midi_out_max_data = 4;
				} else {
					midi_out_state = 0;
				}
				result = MIDI_OUT_BUFFER_STATE_FULL;
			}
		}
	}
	return result;
}

/*
unsigned char pulse_counter = 0;

void USBTxPulse(void) {
	if (!pulse_counter) {
		pulse_counter = 64;
		if (!USBHandleBusy(MIDIDataInHandle)) {
			MIDIDataInHandle = USBTxOnePacket(MIDI_DATA_EP,NULL,0);
		}
	} 
	pulse_counter--;
}
*/

/**********************************************************************************
*
**********************************************************************************/
void clear_usb_hack(void) {
	unsigned char i;

	while ((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1));
	for (i=0;i<64;i++) {
	//for (i=0;i<35;i++) {
		while (USBHandleBusy(MIDIDataInHandle));
    	MIDIDataInHandle = USBTxOnePacket(MIDI_DATA_EP,NULL,0);
	}
}

void USBTxProccessBuffer(void) {
	unsigned char i;

    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
	
	USBMaskInterrupts();
	if (USBHandleBusy(MIDIDataInHandle)) {
		USBUnmaskInterrupts();
		return;
	}
	
	if (midi_out_buffer_full == MIDI_OUT_BUFFER_STATE_SENDING) {
		// Sending empty USB packet.
		// Required. See explanation in USB Specification 2.0: Section 5.8.3
		MIDIDataInHandle = USBTxOnePacket(MIDI_DATA_EP,NULL,0);
		midi_out_buffer_full = MIDI_OUT_BUFFER_STATE_EMPTY;
	}

	if (midi_out_buffer_full == MIDI_OUT_BUFFER_STATE_FULL) {
		midi_out_buffer_full = MIDI_OUT_BUFFER_STATE_SENDING;

		#if NUMBER_MIDI_INTERFACES > 1
		for (i=1;i<NUMBER_MIDI_INTERFACES;i++) {
			midi_out_buffer[i*4] = (midi_out_buffer[0]&0x0f)|(i<<4); 
			midi_out_buffer[i*4 + 1] = midi_out_buffer[1];
			midi_out_buffer[i*4 + 2] = midi_out_buffer[2]; 
			midi_out_buffer[i*4 + 3] = midi_out_buffer[3]; 
		}
		#endif
		for (i=0;i<sizeof(usb_out_buffer);i++) {
			usb_out_buffer[i] = midi_out_buffer[i];
		}
        MIDIDataInHandle = USBTxOnePacket(MIDI_DATA_EP,(BYTE*)&usb_out_buffer,4*NUMBER_MIDI_INTERFACES);
		#if defined(USB_POLLING)
			USBDeviceTasks();
		#endif
	}
	USBUnmaskInterrupts();
}

void USBTxClearRingBuffer(void) {
	while (ring_buffer_outstanding) {
		if (midi_out_buffer_full) break;
		midi_out_buffer_full = MidiStreamToUSB(ring_buffer[ring_buffer_out]);
		ring_buffer_out++;
		ring_buffer_outstanding--;
		if (ring_buffer_out >= sizeof(ring_buffer)) {
			ring_buffer_out = 0;
		}
	}
	if (midi_out_buffer_full) USBTxProccessBuffer();
}

void USBTxMIDI(unsigned char c) {

	//while (ring_buffer_outstanding >= sizeof(ring_buffer)) {
		//This may cause unwanted recursion into USBTxClearRingBuffer()
		//since USBTxClearRingBuffer() is called from main() loop
		//Ring buffer is full but USB is still busy
		//Wait until USB is processing
	//	USBTxClearRingBuffer();
	//}
	if (ring_buffer_outstanding >= sizeof(ring_buffer)) {
		// ring_buffer is full and data are not getting out of it.
		// discard buffer data and start over
		// This shoud never happen really
		ring_buffer_in = 0;
		ring_buffer_out = 0;
		ring_buffer_outstanding = 0;
	}

	ring_buffer[ring_buffer_in] = c;
	ring_buffer_in++;
	ring_buffer_outstanding++;
	if (ring_buffer_in >= sizeof(ring_buffer)) {
		ring_buffer_in = 0;
	}
}

unsigned char USBRxMIDI(void) {
	return midi_data_rx[midi_data_rx_pointer++];
}

unsigned char USBRxEmpty(void) {
	//unsigned char result = 0;

	if (midi_data_rx_pointer < midi_in_data_max) {
		return 0;
	}
	//if (midi_data_rx_pointer > 0) {
	//	return 1;
	//}
	if (midi_in_received && (midi_data_rx_pointer < midi_rx_len)) {			
		midi_in_received = 0;
		switch(midi_data_rx[midi_data_rx_pointer]&0x0f) {
			case 0x05:
			case 0x0f:
				midi_in_data_max = midi_data_rx_pointer + 2;
				break;
			case 0x02:
			case 0x06:
			case 0x0c:
			case 0x0d:
				midi_in_data_max = midi_data_rx_pointer + 3;
				break;
			case 0x04:
				midi_in_received = 1;
				midi_in_data_max = midi_data_rx_pointer + 4;
				break;
			case 0x03:
			case 0x07:
			case 0x08:
			case 0x09:
			case 0x0a:
			case 0x0b:
			case 0x0e:
				midi_in_data_max = midi_data_rx_pointer + 4;
				break;
			default:
				midi_in_data_max = midi_data_rx_pointer + 1;
				break;
		}
		midi_data_rx_pointer++;
		return 0;
	} else {
		return 1;
	}
}

unsigned char USBRxReady(void) {
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return 0;
	
	if (!USBRxEmpty()) {
		return 1;
	}

	midi_in_data_max = 0;
	midi_data_rx_pointer = 0;
	midi_rx_len = 0;
	
    if(!USBHandleBusy(MIDIDataOutHandle))
    {
		midi_data_rx_len = USBHandleGetLength(MIDIDataOutHandle);
		if (midi_data_rx_len>MIDI_DATA_OUT_EP_SIZE) {
			midi_data_rx_len = MIDI_DATA_OUT_EP_SIZE;
		}

        for(midi_rx_len = 0; midi_rx_len < midi_data_rx_len; midi_rx_len++)
            midi_data_rx[midi_rx_len] = usb_data_rx[midi_rx_len];

        MIDIDataOutHandle = USBRxOnePacket(MIDI_DATA_EP,(BYTE*)&usb_data_rx,sizeof(usb_data_rx));
		midi_in_received = 1;
		//return 1;
	}
	return 0;
}
